using System;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;


namespace VirtualListView 
{
    class MyVirtualListView : ListView
    {
        private ListViewItem[] _itemCache;

        public MyVirtualListView() : base()
        {
            SetStyle(ControlStyles.DoubleBuffer, true);
            //SetStyle(ControlStyles.UserMouse, true);

            this.OwnerDraw = true;
            this.VirtualMode = true;
            this.DoubleBuffered = true;
            this.View = View.Details;
            this.FullRowSelect = true;
            this.VirtualListSize = 1000000;
            
            this._itemCache = new ListViewItem[this.VirtualListSize];

            this.Columns.Add("item", 120, HorizontalAlignment.Left);
            this.Columns.Add("subitem1", 120, HorizontalAlignment.Left);
            this.Columns.Add("subitem2", 120, HorizontalAlignment.Left);
        }
        
        protected override void OnDrawItem(DrawListViewItemEventArgs e)
        {
            e.DrawBackground();
            e.DrawFocusRectangle();

            if ((e.State & ListViewItemStates.Selected) != 0)
            {
                // Draw the background and focus rectangle for a selected item.
                e.Graphics.FillRectangle(Brushes.Blue, e.Bounds);
                e.DrawFocusRectangle();
            }
            else
            {
                // Draw the background for an unselected item.
                e.Graphics.FillRectangle(Brushes.White, e.Bounds);
            }

            // Draw the item text for views other than the Details view.
            if (this.View != View.Details)
            {
                e.DrawText();
            }
        }

        // Draws subitem text and applies content-based formatting.
        protected override void OnDrawSubItem(DrawListViewSubItemEventArgs e)
        {
            TextFormatFlags flags = TextFormatFlags.Left;
            using (StringFormat sf = new StringFormat())
            {
                // Store the column text alignment, letting it default
                // to Left if it has not been set to Center or Right.
                switch (e.Header.TextAlign)
                {
                    case HorizontalAlignment.Center:
                        sf.Alignment = StringAlignment.Center;
                        flags = TextFormatFlags.HorizontalCenter;
                        break;
                    case HorizontalAlignment.Right:
                        sf.Alignment = StringAlignment.Far;
                        flags = TextFormatFlags.Right;
                        break;
                }
                Rectangle bounds = e.Bounds;
                e.Graphics.DrawString(e.SubItem.Text, this.Font, Brushes.Black, e.Bounds, sf);
            }
        }
        protected override void OnDrawColumnHeader(DrawListViewColumnHeaderEventArgs e)
        {
            using (StringFormat sf = new StringFormat())
            {
                // Store the column text alignment, letting it default
                // to Left if it has not been set to Center or Right.
                switch (e.Header.TextAlign)
                {
                    case HorizontalAlignment.Center:
                        sf.Alignment = StringAlignment.Center;
                        break;
                    case HorizontalAlignment.Right:
                        sf.Alignment = StringAlignment.Far;
                        break;
                }

                // Draw the standard header background.
                e.DrawBackground();

                // Draw the header text.
                using (Font headerFont = new Font("Verdana", 8, FontStyle.Regular))
                {
                    e.Graphics.DrawString(e.Header.Text, headerFont,Brushes.Black, e.Bounds, sf);
                }
            }
            return;
        }
        protected override void OnRetrieveVirtualItem(RetrieveVirtualItemEventArgs e)
        {
            ListViewItem itm = _itemCache[e.ItemIndex];

            if (itm == null)
            {
                itm = new ListViewItem("Item_" + e.ItemIndex.ToString());
                itm.SubItems.Add("subitem1_" + e.ItemIndex.ToString());
                itm.SubItems.Add("subitem2_" + e.ItemIndex.ToString());
            }
            e.Item = itm;
        }

        //protected override void OnMouseMove(MouseEventArgs e)
        //{
        //    //this.Invalidate();
        //    ListViewItem Item = this.GetItemAt(5, e.Y);
        //    if (Item != null)
        //    {
        //        Item.ListView.Invalidate(Item.GetBounds(ItemBoundsPortion.Entire));
        //        Item.Focused = true;
        //    }
        //}
    }
}
